package com.example.ellegadodepintia.exploradoresDePintia

import android.annotation.SuppressLint
import android.app.Activity
import android.content.Context
import android.content.Intent
import android.graphics.Color
import android.graphics.drawable.AnimationDrawable
import android.os.Bundle
import android.os.Handler
import android.os.Looper
import android.view.MotionEvent
import android.view.View
import android.widget.GridLayout
import android.widget.ImageButton
import android.widget.ImageView
import android.widget.LinearLayout
import android.widget.ProgressBar
import android.widget.TextView
import android.widget.Toast
import androidx.activity.result.ActivityResultLauncher
import androidx.activity.result.contract.ActivityResultContracts
import androidx.appcompat.app.AppCompatActivity
import com.example.ellegadodepintia.R
import com.example.ellegadodepintia.exploradoresDePintia.model.Atributo
import com.example.ellegadodepintia.exploradoresDePintia.model.GameState

import com.example.ellegadodepintia.exploradoresDePintia.model.GeneradorDeEventos
import com.example.ellegadodepintia.exploradoresDePintia.model.LayoutUtils
import com.example.ellegadodepintia.exploradoresDePintia.model.GestorDeEventos
import com.example.ellegadodepintia.repositorios.RepositorioJugador
import com.example.ellegadodepintia.repositorios.RepositorioObjetos
import com.example.ellegadodepintia.soundManager.SoundManager
import java.lang.ref.WeakReference
import java.util.ArrayList

class ExploradoresDePintia : AppCompatActivity() {
    private lateinit var soundManager: SoundManager
    private var dificultad = 0
    private lateinit var startForResult: ActivityResultLauncher<Intent>


    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        //RepositorioJugador.setInventario(ArrayList())
        //Habria que vaciar de la base de datos el inventario del jugador, por si se sale, que pierda
        //todo el inventario, al terminar la partida, si queremos guardar algun objeto, se hace al final una
        //carga a la base de datos, habria que cambiar todos los metodos que dan objetos al personaje
        //para que solo se guarden en local.
        setContentView(R.layout.activity_exploradores_de_pintia)

        soundManager = SoundManager(this)
        soundManager.playSoundLoop(R.raw.sound_exploradores,50)
        //soundManager.playSoundLoop(R.raw.sound_exploradores_idle,100)
        startForResult =
            registerForActivityResult(ActivityResultContracts.StartActivityForResult()) { result ->
                if (result.resultCode == Activity.RESULT_OK) {
                    val mensajeResultado = result.data?.getStringExtra("resultado")
                    println("Volvemos del mini-juego: $mensajeResultado")
                }
            }

        GameState.fragmentManager = supportFragmentManager

        GeneradorDeEventos.generarEventos()

        GameState.setContexto(WeakReference<Context>(this))

        val inventoryButton: ImageButton = findViewById(R.id.inventoryButton)
        val eventButton: ImageButton = findViewById(R.id.eventButton)

        eventButton.setBackgroundResource(R.drawable.style_texto_border)
        inventoryButton.setBackgroundColor(Color.TRANSPARENT)

        inventoryButton.setOnClickListener {
            val inventarioFragment = InventoryMain()
            inventarioFragment.show(supportFragmentManager, "inventario")
        }

        val imagenProtagonista = findViewById<ImageView>(R.id.imagenProtagonista)
        val idleAnimation = imagenProtagonista.drawable as? AnimationDrawable
        idleAnimation?.start()

        iniciarJuego()
    }

    private fun iniciarJuego() {
        GameState.jugador.addObserver {
            LayoutUtils.actualizarEstadisticasMain()
            LayoutUtils.actualizarMonedas()
        }
        LayoutUtils.actualizarEstadisticasMain()
        LayoutUtils.actualizarMonedas()
        GeneradorDeEventos.generarEvento(dificultad)
        GestorDeEventos.gestionar()
        ListenerBarras()

    }

    @Deprecated("Deprecated in Java")
    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?) {
        super.onActivityResult(requestCode, resultCode, data)
        if (requestCode == 100 && resultCode == RESULT_OK) {
            val resultado = data?.getStringExtra("resultado")
            Toast.makeText(this, resultado, Toast.LENGTH_SHORT).show()

        } else if (requestCode == 102 && resultCode == RESULT_OK) {
            val puntuacion = data?.getStringExtra("PUNTUACION")!!.toInt()
            GameState.jugador.actualizarAtributo(Atributo.Energia, puntuacion/3)
            GameState.jugador.actualizarAtributo(Atributo.Vida, puntuacion/3)

        } else if (requestCode == 103 && resultCode == RESULT_OK) {
            val tiempo = data?.getStringExtra("TIEMPO")!!.toInt()
            if (tiempo >= 1000){
                GameState.jugador.actualizarMonedas(GameState.jugador.monedas.plus(tiempo/1000))
            }else if (GameState.jugador.inventario.size == 0){
                if (GameState.jugador.atributos[Atributo.Energia] == 1){
                    GameState.jugador.actualizarAtributo(Atributo.Energia, -1)
                }else{
                    GameState.jugador.actualizarAtributo(Atributo.Energia, -GameState.jugador.atributos[Atributo.Energia]!!/2)
                }
            }else{
                GameState.jugador.destruirObjetoRandom()
            }
        }else if (requestCode == 104 && resultCode == RESULT_OK) {
            val puntosHuesos = data?.getStringExtra("PUNTOS_HUESOS")!!.toInt()
            val objeto= RepositorioObjetos.obtenerObjetoPorCoste(puntosHuesos)
            if (objeto != null){
                GameState.jugador.conseguirObjeto(objeto)
            }
        }

    }

    override fun onDestroy() {
        super.onDestroy()
        soundManager.stopSound()
    }

    @SuppressLint("ClickableViewAccessibility")
    private fun ListenerBarras() {
        val barras: LinearLayout = findViewById(R.id.barras)
        val estadisticas: GridLayout = findViewById(R.id.estadisticas)
        barras.setOnTouchListener { v, event ->
            when (event.action) {
                MotionEvent.ACTION_DOWN -> {
                    Handler(Looper.getMainLooper()).postDelayed({
                        estadisticas.visibility = View.VISIBLE
                    }, 500) // 500 ms para que se registre como toque largo
                }
                MotionEvent.ACTION_UP, MotionEvent.ACTION_CANCEL -> {
                    // Ocultar el GridLayout si se termina el toque
                    estadisticas.visibility = View.GONE
                }
            }
            true
        }
    }
    override fun onBackPressed() {
        val flag = intent.getBooleanExtra("FLAG_KEY", false)
        if (flag){
            super.onBackPressed()
        }
    }
    override fun onPause() {
        super.onPause()
        soundManager.setVolume(0)
    }

    override fun onResume() {
        super.onResume()
        soundManager.setVolume(60)

    }
}
